@@ -844,9 +844,11 @@ bool calculate_userfunctions(MathStructure &m, const MathStructure &x_mstruct, c
 		}
 	}
 	if(m.isFunction()) {
-		if(!m.contains(x_mstruct, true)) {
-			m.calculateFunctions(eo);
-			b_ret = true;
+		if(!m.contains(x_mstruct, true) && !m.containsFunctionId(FUNCTION_ID_RAND, true, true, true) && !m.containsFunctionId(FUNCTION_ID_RANDN, true, true, true) && !m.containsFunctionId(FUNCTION_ID_RAND_POISSON, true, true, true)) {
+			if(m.calculateFunctions(eo, false)) {
+				b_ret = true;
+				calculate_userfunctions(m, x_mstruct, eo, b_vector);
+			}
 		} else if(m.function()->subtype() == SUBTYPE_USER_FUNCTION && m.function()->condition().empty()) {
 			bool b = true;
 			for(size_t i = 0; i < ((UserFunction*) m.function())->countSubfunctions(); i++) {
